from collections import deque


def structure_parser(obj):
    structure = {}
    queue = deque([(k, v) for k, v in obj.items()])
    while len(queue) != 0:
        key, value = queue.popleft()
        structure[key] = list()
        if isinstance(value, dict):
            for k, v in value.items():
                structure[key].append(k)
                queue.append((k, v))

    return structure


def printer(structure, key, level, visited):
    if key in visited:
        return
    visited.add(key)
    print("    " * level, key)
    value = structure.get(key)
    if len(value) == 0:
        return
    else:
        for v in value:
            printer(structure, v, level + 1, visited)


def structure_printer(structure):
    visited = set()
    for k, v in structure.items():
        printer(structure, k, 0, visited)


def structure_summary(response):
    if isinstance(response, dict):
        structure = structure_parser(response)
        structure_printer(structure)
    if isinstance(response, list):
        structure = structure_parser(response[0])
        structure_printer(structure)



import pytest

from dataray.helper.helper_func import structure_parser
from dataray.helper.helper_func import structure_printer

@pytest.fixture()
def flatten_dict():
    return {"a": 1, "b": 1, "c": 1}


@pytest.fixture()
def nested_dict():
    return {"a": {"b": {"f": 1}, "c": 1, "d": 1}, "e": 1}


@pytest.fixture()
def parsed_structure():
    return {"a": ["d", "f"], "b": [], "c": [], "d": [], "f": []}


def test_structure_parser_flatten_dict(flatten_dict):
    parsed = structure_parser(flatten_dict)
    expected = {"a": [], "b": [], "c": []}
    assert parsed == expected


def test_structure_parser_nested_dict(nested_dict):
    parsed = structure_parser(nested_dict)
    expected = {"a": ["b", "c", "d"], "b": ["f"], "c": [], "d": [], "e": [], "f": []}
    assert parsed == expected


def test_structure_printer(parsed_structure, capsys):
    structure_printer(parsed_structure)
    captured = capsys.readouterr()
    assert captured.out == " a\n     d\n     f\n b\n c\n"



from dataray.helper import structure_summary


class DataRay:
    def ray(self):
        def wrapper(request_func, *args, **kwargs):
            response = request_func(*args, **kwargs)
            structure_summary(response)
            return request_func
        return wrapper







# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dataray', 'dataray.helper', 'dataray.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'dataray',
    'version': '0.1.0',
    'description': 'a class of decorators helping inspect data',
    'long_description': '# DataRay\n\nDataRay is a class of decorators helping detect the metadata of the data.\n\nThe motivation of this package is to help understand how your api request data looks like. Normally we can directly\ncheck how the api request looks like via some UI platform such as postman etc. With this package, user may directly\nget the data structure and some basic metadata of the requested data\n\n## Features\n* Json Structure\n* Further feature coming soon\n\n\n## Detect the Json Structure\n\n```python\n# here is your customer request function. Now it is supposed to return list or dict you are interested in looking into\nfrom dataray import dataray\n\n@ dataray.ray()\ndef request_func():\n    ...\n```\nThen every time calling `request_func`, the json structure will be printed out.',
    'author': 'Zhen404',
    'author_email': 'zl2632@columbia.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Zhen404/dataray',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
